using System.Collections.Generic;
using System.Linq;

namespace More.Engine.Model
{
    /// <summary>
    /// This is a model class for a rating table.  It contains the column and key definitions
    /// </summary>
    public class LookupTable
    { 
		#regionFields(1)

        private List<LookupTableRow> _rows;

		#endregionFields

		#regionConstructors(3)

        public LookupTable(string name, FactorTableColumnDefinition[] keys, FactorTableColumnDefinition[] columns)
        {
            Name = name;
            Keys = keys;
            Columns = columns;
        }

        public LookupTable(string name)
        {
            Name = name;
        }

        public LookupTable()
        {
        }

		#endregionConstructors

		#regionProperties(4)

        /// <summary>
        /// The Column/Value Definitions
        /// </summary>
        public FactorTableColumnDefinition[] Columns { get; set; }

        /// <summary>
        /// The keys definition
        /// </summary>
        public FactorTableColumnDefinition[] Keys { get; set; }

        public FactorTableColumnDefinition[] KeysAndColumns
        {
            get { return Keys.Concat(Columns).ToArray(); }
        }

        /// <summary>
        /// The name of the tables
        /// </summary>
        public string Name { get; set; }
        
        /// <summary>
        /// The actual column & key values the makes a row of data
        /// </summary>
        public List<LookupTableRow> Rows
        {
            get { return _rows ?? (_rows = new List<LookupTableRow>()); }
            set { _rows = value; }
        }
        
		#endregionProperties

        public string Description
        {
            get
            {
                return string.Format("{0} ( columnName, {1} )", Name, string.Join(", ", Keys.Select(p => p.Name)));
            }
            
        }

        private Dictionary<string, object> _properties;
        public Dictionary<string, object> Properties
        {
            get { return _properties ?? (_properties = new Dictionary<string, object>()); }
            set { _properties = value; }
        }

        public bool IsList { get; set; }

        public string Id { get; set; }

        public override string ToString()
        {
            return Description;
        }

    }
}