using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using More.Engine.BaseModel;

namespace More.Engine.Model
{
    [Serializable]
    public class InputRequirement
    {
        public string Name { get; set; }
    }
    /// <summary>
    /// The final result of a rating engine invokation.
    /// It implements IRuleBookTraceInformation because it 
    /// has a list of Trace Information
    /// </summary>
    [Serializable]
    public class RuleBookResult : IRuleBookTraceInformation
    {
        #regionFields(3)

        private List<RuleBookError> _errors;
        [IgnoreDataMember]
        private Dictionary<string, object> _results;
        private List<RuleBookTraceInformation> _traceInformation;

        #endregionFields
        
        #regionConstructors(3)

        public RuleBookResult(Dictionary<string, object> results, List<RuleBookError> errors)
        {
            Results = results;
            Errors = errors;
        }

        public RuleBookResult(Dictionary<string, object> results)
        {
            Results = results;
        }

        public RuleBookResult()
        {
        }

        #endregionConstructors

        #regionProperties(4)
        [DataMember]
        public List<RuleBookError> Errors
        {
            get { return _errors ?? (_errors = new List<RuleBookError>()); }
            set { _errors = value; }
        }
       [IgnoreDataMember]
        
        public Dictionary<string, object> Results
        {
            get { return _results ?? (_results = new Dictionary<string, object>()); }
            set { _results = value; }
        }

        [DataMember]
        public List<RuleBookTraceInformation> TraceInformation
        {
            get { return _traceInformation ?? (_traceInformation = new List<RuleBookTraceInformation>()); }
            set { _traceInformation = value; }
        }

        public object Find(string name)
        {
            return Find(TraceInformation, name);
        }
        public object Find(IEnumerable<RuleBookTraceInformation> info, string name)
        {
            foreach (var child in info)
            {
                if (child.Description.ToUpper() == name.ToUpper())
                {
                    return child.Value;
                }
                var result = Find(child.TraceInformation, name);
                if (result != null)
                {
                    return result;
                }
            }
            return null;
        }
        #endregionProperties

    }


}