using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using More.Engine.BaseModel;
using More.Engine.Evaluation;

namespace More.Engine
{
    /// <summary>
    /// The rating engine WOOT WOOT! Its primary role is to keep track of the rating assemblies 
    /// including the domain at which these assemblies are executed under
    /// </summary>
    public class RulesEngine
    {
		#regionConstructors(1)

        public RulesEngine()
        {
            if (RatingDomain != null)
                Flush();
            else
                Initialize();
        }

		#endregionConstructors

		#regionProperties(3)

        public IEnumerable<Assembly> Assemblies
        {
            get { return RatingDomain.GetAssemblies(); }
        }

        public static AppDomain RatingDomain { get; set; }

		#endregionProperties

		#regionMethods(3)

		//PublicMethods(2)

        public void Flush()
        {
            AppDomain.Unload(RatingDomain);
            Initialize();
        }
      
        public CompiledRuleBookFactoryBase GetFactory(string name)
        {
          
            try
            {
                var assembly = RatingDomain.Load(name);

                var rater = (CompiledRuleBookFactoryBase)RatingDomain.CreateInstanceAndUnwrap(name, name + ".CompiledRuleBookFactory");

                return rater;
            }
            catch (Exception ex)
            {
                return null;
            }

        }
		//PrivateMethods(1)

        private void Initialize()
        {

            var setup = new AppDomainSetup
                            {
                                ApplicationName = "MoreEngine",
                                ApplicationBase = AppDomain.CurrentDomain.BaseDirectory,
                                PrivateBinPath = AppDomain.CurrentDomain.SetupInformation.PrivateBinPath,
                                CachePath = AppDomain.CurrentDomain.SetupInformation.CachePath,
                                ShadowCopyFiles = "true",
                                ShadowCopyDirectories = AppDomain.CurrentDomain.SetupInformation.ShadowCopyDirectories
                            };

            RatingDomain = AppDomain.CreateDomain(
                "RatingEngine", null, setup);



            foreach (var assembly in AppDomain.CurrentDomain.GetAssemblies().Where(p => p.FullName.StartsWith("System"))
                )
                RatingDomain.Load(assembly.FullName);

            RatingDomain.Load(new AssemblyName("More.Engine"));
        }

		#endregionMethods
    }
}