﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using More.Application.BaseModel;
using More.Engine.Compiling;
using More.Engine.Model;

namespace More.Web.Designer.Controllers
{
    public class RulesController : BaseController
    {
        protected override void Initialize(System.Web.Routing.RequestContext requestContext)
        {
            base.Initialize(requestContext);
            ViewBag.CurrentRuleBook = Strategy.GetRuleBook(CurrentRuleBookId).Name;
            ViewBag.CurrentRuleBookId = CurrentRuleBookId;
        }
        public ActionResult Index(int? ruleBookId = null, int? id = null, string parentId = null)
        {
            if (ruleBookId != null)
            {
                CurrentRuleBookId = ruleBookId ?? 0;
                ViewBag.CurrentRuleBook = Strategy.GetRuleBook(CurrentRuleBookId).Name;
                ViewBag.CurrentRuleBookId = CurrentRuleBookId;
            }
            if (id != null)
            {
                return View("Rule", Strategy.GetRule(id ?? 0) ?? new RuleEngineRule("0", "NewRule")
                                                            {
                                                                ParentChangeId =
                                                                    string.IsNullOrEmpty(parentId)
                                                                        ? (Guid?) null
                                                                        : new Guid(parentId),
                                                                ChangeId = Guid.NewGuid().ToString(),
                                                                EffectiveDate = MissionControlDate,
                                                                Active = true
                                                            });
            }
            return View(RuleHelper.SortRules(Strategy.GetRuleBookRules(CurrentRuleBookId).ToArray()));
        }
        public ActionResult StepIntoRule(string id)
        {
            StepInto = id;
            ViewBag.StepInto = id;
            return View("Index", RuleHelper.SortRules(Strategy.GetRuleBookRules(CurrentRuleBookId).ToArray()));
        }
        public ActionResult SaveRule(RuleEngineRule model)
        {
            model.RuleBookId = CurrentRuleBookId;
            Strategy.SaveRule(model);
            return View("Index", RuleHelper.SortRules(Strategy.GetRuleBookRules(CurrentRuleBookId).ToArray()));
        }
        public ActionResult RuleExpression(int ruleId)
        {
            return Content(Strategy.GetRule(ruleId).RuleExpression);
        }
        public ActionResult SaveRuleExpression(int ruleId, string expression)
        {
            var rule = Strategy.GetRule(ruleId);
            rule.RuleExpression = expression;
            Strategy.SaveRule(rule);
            return View("Index", RuleHelper.SortRules(Strategy.GetRuleBookRules(CurrentRuleBookId).ToArray()));
        }
        public ActionResult DeleteRule(int ruleId)
        {
            Strategy.DeleteRule(ruleId);
            return View("Index", RuleHelper.SortRules(Strategy.GetRuleBookRules(CurrentRuleBookId).ToArray()));
        }
        public ActionResult AssignRuleParent(int ruleId, string parentId)
        {
            var rules = Strategy.GetRuleBookRules(CurrentRuleBookId).ToArray();
            foreach (var ruleEngineRule in rules)
            {
                var parent = ruleEngineRule.Find(parentId);
                if (parent == null) continue;
                var x = parent.Parents.FirstOrDefault(p => p.Id == ruleId.ToString());
                if (x != null)
                {
                    return View("Index", rules);
                }
                break;
            }
            var rule = Strategy.GetRule(ruleId);
            rule.ParentChangeId = string.IsNullOrEmpty(parentId) ? (Guid?)null : new Guid(parentId);
            Strategy.SaveRule(rule);
            return View("Index", RuleHelper.SortRules(Strategy.GetRuleBookRules(CurrentRuleBookId).ToArray()));
        }



    }
}
