using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;

namespace More.Application.Entity
{
    public static class FlatTableQueries
    {
        public static string DeleteRowScript(string tableName, string rowId)
        {
            return string.Format("DELETE FROM [{0}] WHERE Id = {1}", tableName, rowId);
        }

        public static string InsertRowScript(string tableName, string[] columns, string[] values, string changeId, DateTime effectiveDate, bool active)
        {
            return string.Format(
                "INSERT INTO [{0}] ( [ChangeId],[EffectiveDate], [Active], {1} ) VALUES ( '{2}', '{3}', '{4}', {5} )",
                tableName,
                ColumnsFormated(columns),
                changeId,
                effectiveDate.ToString(CultureInfo.InvariantCulture),
                active ? "1" : "0",
                string.Join(",", values.Select(p => "'" + p + "'"))
                );
        }
        
        public static string UpdateRowScript(long rowId, string tableName, string[] columns, string[] values, string changeId, DateTime effectiveDate, bool active)
        {
            var sb = new StringBuilder();
            sb.AppendFormat("UPDATE {0} SET ", tableName);
            sb.AppendFormat("ChangeId = '{0}', EffectiveDate = '{1}', Active = {2} ", changeId, effectiveDate.ToString(CultureInfo.InvariantCulture), active ? "1" : "0");
            for (int i = 0; i < columns.Length; i++)
            {
                sb.AppendFormat(", {0} = '{1}'", columns[i], values[i]);
            }
            sb.AppendFormat(" WHERE Id = {0}", rowId);
            return sb.ToString();
        }
      
        public static string SelectSingle(string table, string[] columns, string id)
        {
            return string.Format("SELECT [Id], [ChangeId], [EffectiveDate], [Active], {0} FROM [dbo].[{1}] where Id = {2}",
                                 ColumnsFormated(columns), table, string.IsNullOrEmpty(id) ? "0" : id);
        }

        public static string BuildVersionedQuery(string table, string[] columns, DateTime effectiveDate)
        {
            var sb = new StringBuilder();
            sb.AppendFormat("SELECT [Id],[ChangeId], [EffectiveDate], [Active], {0} FROM [dbo].[{1}] ft ", ColumnsFormated(columns), table).AppendLine();
            sb.AppendLine("WHERE ft.[Id] IN (");
            sb.AppendFormat("SELECT MAX(ft1.[Id]) [Id] FROM [dbo].[{0}] ft1", table);
            sb.AppendLine(" WHERE ft1.[Id] IN (");
            sb.AppendLine("SELECT ft2.[Id] [Id]");
            sb.AppendFormat("FROM [dbo].[{0}] ft2 ", table);
            sb.AppendFormat("WHERE ft2.[EffectiveDate] <= '{0}'", effectiveDate.ToString());
            sb.AppendLine(")");
            sb.AppendLine("GROUP BY ft1.[ChangeId]");
            sb.AppendLine(")");

            return sb.ToString();
        }
        public static string UpdateColumnExtendedProperty(string tableName,string columnName, string name, string value)
        {
            return string.Format(
                "EXEC sys.sp_updateextendedproperty @name=N'{0}', @value=N'{1}' , @level0type=N'SCHEMA',@level0name=N'dbo', @level1type=N'TABLE',@level1name=N'{2}', @level2type=N'COLUMN',@level2name=N'{3}'",
                    name, value, tableName, FixColumnName(columnName));
        }
        public static string FixColumnName(string name)
        {
            return name;
        }
        public static string BuildVersionedQueryCount(string table, DateTime effectiveDate)
        {
            var sb = new StringBuilder();
            sb.AppendFormat("SELECT count(*) FROM [dbo].[{0}] ft ", table).AppendLine();
            sb.AppendLine("WHERE ft.[Id] IN (");
            sb.AppendFormat("SELECT MAX(ft1.[Id]) [Id] FROM [dbo].[{0}] ft1", table);
            sb.AppendLine(" WHERE ft1.[Id] IN (");
            sb.AppendLine("SELECT ft2.[Id] [Id]");
            sb.AppendFormat("FROM [dbo].[{0}] ft2 ", table);
            sb.AppendFormat("WHERE ft2.[EffectiveDate] <= '{0}'", effectiveDate.ToString());
            sb.AppendLine(")");
            sb.AppendLine("GROUP BY ft1.[ChangeId]");
            sb.AppendFormat(")").AppendLine();

            return sb.ToString();
        }

        private static string ColumnsFormated(IEnumerable<string> columns)
        {
            return string.Join(",", columns.Select(p => "[" + p + "]"));
        }
    }
}