﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using More.Application.BaseModel;
using More.Application.Entity;
using More.Engine.CodeGen.Templates.Model;
using More.Engine.Compiling;
using More.Engine.Model;
using More.Engine.Validation;

namespace More.Application
{
    public static class MoreApplicationFactory
    {

        public static ILookupTablesRepository GetLookupRepository(DateTime effectiveDate, bool setupMode = false)
        {
            return new FlatTableRatingConfigStrategy(effectiveDate, new MoreLookupTableEntities())
            {
                SetupMode = setupMode
            };
        }
        public static IRulesEngineRepository GetEngineRepository(DateTime effectiveDate, bool setupMode = false)
        {
            //EntityRulesEngineRepository.Validator = new RatingRuleValidator()
            //                                            {
                                                            
            //                                            };
            return new EntityRulesEngineRepository(effectiveDate, new MoreEntities())
            {   
                SetupMode = setupMode
            };
        }

        public static RuleEngineCodeModel GetRuleEngineCodeModel(string nameSpace, DateTime effectiveDate, string[] ruleBookIds = null)
        {
            var lookupRepository = GetLookupRepository(effectiveDate);
            var engineRepository = GetEngineRepository(effectiveDate);
            RuleBookTemplateModel[] ruleBooks = null;

            if (ruleBookIds != null)
            {
                ruleBooks = engineRepository.GetRuleBooks().Where(p => ruleBookIds.Contains(p.Id)).Select(
                           p => new RuleBookTemplateModel(
                                        p.Name, "CommonRuleBook",
                                        RuleHelper.SortRules(engineRepository.GetRuleBookRules(Convert.ToInt32((string)p.Id), null).ToArray()).ToArray(),
                                        null
                                    )
                        ).ToArray();
            }
            else
            {
                ruleBooks = engineRepository.GetRuleBooks().Select(
                           p => new RuleBookTemplateModel(
                                        p.Name, "CommonRuleBook",
                                        RuleHelper.SortRules(engineRepository.GetRuleBookRules(Convert.ToInt32((string)p.Id), null).ToArray()).ToArray(),
                                        null
                                    )
                        ).ToArray();
            }


            var model = new RuleEngineCodeModel()
                            {
                                Namespace = nameSpace,
                                RuleBooks = ruleBooks,
                                BaseLookupTables = lookupRepository.GetRatingTables().
                                    Select(p => lookupRepository.GetLookupTable(p.Id)).ToArray()
                            };
            return model;
        }

        public static RatingEngineCompiler GetCompiler( DateTime effectiveDate)
        {
            var compiler = new RatingEngineCompiler(GetRuleEngineCodeModel(GetAssemblyName(effectiveDate), effectiveDate));

            return compiler;
        }
        public static DateTime GetAssemblyEffectiveDate(DateTime effectiveDate)
        {
            var assemblyEffectiveDate = GetEngineRepository(effectiveDate).GetRatingAssemblies().Where(p => p.EffectiveDate <= effectiveDate).
                OrderByDescending(p => p.EffectiveDate).FirstOrDefault();
            if (assemblyEffectiveDate != null)
            {
                return assemblyEffectiveDate.EffectiveDate;
            }
            throw new Exception("An assembly for the effective date " + effectiveDate.ToString() + "Couldn't  be found. Make sure it has been compiled and published.");
        }
        public static string GetAssemblyName(DateTime effectiveDate)
        {
            
            return string.Format("Rules{0}{1}{2}", effectiveDate.Month, effectiveDate.Day,
                                 effectiveDate.Year);
        }
    }
}
