﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Web;
using System.Web.Http;
using More.Application;
using More.Engine;
using More.Engine.Model;
using More.Services.WebAPI.Models;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
namespace More.Services.WebAPI.Controllers
{
    public class ValuesController : ApiController
    {
        public static Dictionary<string, object> FromJson(string json)
        {
            var jsonReader = new JsonTextReader(new StringReader(json));
            var inputs = new Dictionary<string, object>();
            ReadData(jsonReader, inputs, null);
            return inputs;
        }
        public static void ReadData(JsonTextReader reader, Dictionary<string, object> inputs, List<Dictionary<string, object>> currentList)
        {
            var propertyName = string.Empty;
            while (reader.Read())
            {
                switch (reader.TokenType)
                {
                    case JsonToken.StartObject:
                        if (currentList == null)
                        {
                            if (string.IsNullOrEmpty(propertyName))
                            {
                                ReadData(reader, inputs, null);
                            }
                            else
                            {
                                var objectDictionary = new Dictionary<string, object>();
                                inputs.Add(propertyName, objectDictionary);
                                ReadData(reader, objectDictionary, null);
                            }
                        }
                        else
                        {
                            var objectDictionary = new Dictionary<string, object>();
                            currentList.Add(objectDictionary);
                            ReadData(reader, objectDictionary, currentList);
                        }
                        break;
                    case JsonToken.StartArray:
                        var list = new List<Dictionary<string, object>>();
                        ReadData(reader, inputs, list);
                        inputs.Add(propertyName, list);
                        break;
                    case JsonToken.EndObject:
                        return;
                    case JsonToken.EndArray:
                        return;
                        break;
                    case JsonToken.Boolean:
                        inputs.Add(propertyName, Convert.ToBoolean(reader.Value));
                        break;
                    case JsonToken.Float:
                        inputs.Add(propertyName, Convert.ToDouble(reader.Value));
                        break;
                    case JsonToken.Integer:
                        inputs.Add(propertyName, Convert.ToDouble(reader.Value));
                        break;
                    case JsonToken.Bytes:
                        inputs.Add(propertyName, Convert.ToInt32(reader.Value));
                        break;
                    case JsonToken.Date:
                        DateTime dateValue;
                        Int32 intValue;

                        if (DateTime.TryParse(reader.Value.ToString(), out dateValue))
                        {
                            inputs.Add(propertyName, dateValue);
                        }
                        else if (Int32.TryParse(reader.Value.ToString(), out intValue))
                        {
                            inputs.Add(propertyName, intValue);
                        }
                        else
                        {
                            inputs.Add(propertyName, reader.Value.ToString());
                        }
                        
                        break;
                    case JsonToken.Null:
                        inputs.Add(propertyName, string.Empty);
                        break;
                    case JsonToken.String:
                        inputs.Add(propertyName, reader.Value.ToString());
                        break;
                    case JsonToken.PropertyName:
                        propertyName = reader.Value.ToString();
                        break;
                }

            }
        }
        //public static string WriteData(IEnumerable<RuleBookTraceInformation> trace)
        //{
        //    var sb = new StringBuilder();
        //    var writer = new JsonTextWriter(new StringWriter(sb));

        //    writer.WriteStartObject();
        //    WriteData(writer, trace);
        //    writer.WriteEndObject();
        //}
        public static string GetJsonTrace(Dictionary<string,object> trace)
        {
            var sb = new StringBuilder();
            var writer = new JsonTextWriter(new StringWriter(sb));
            writer.WriteStartObject();
            WriteList(writer,trace);
            writer.WriteEndObject();
            return sb.ToString();
        }
        public static void WriteList(JsonTextWriter writer, Dictionary<string,object> items)
        {
            //writer.WriteStartArray();
            foreach (var item in items)
            {
                writer.WritePropertyName(item.Key);
                if (item.Value is IList<Dictionary<string,object>>)
                {
                    var itemList = item.Value as IList<Dictionary<string, object>>;
                    writer.WriteStartArray();
                    foreach (var x in itemList)
                    {
                        WriteList(writer, x);
                    }
                   
                    writer.WriteEndArray();
                } else if (item.Value is Dictionary<string,object>)
                {
                    writer.WriteStartObject();
                    WriteList(writer,item.Value as Dictionary<string,object>);
                    writer.WriteEndObject();
                } 
                
                else
                {
                    writer.WriteValue(item.Value);
                }
                

            }
           // writer.WriteEndArray();
        }
        
        //public RuleBookTraceInformation[] Execute(DateTime effectiveDate, string ruleBookName, Dictionary<string, object> inputs)
        //{

        //    //MoreApplicationFactory.GetCompiler(effectiveDate);
        //    var rulesEngine = new RulesEngine();
        //    var factory = rulesEngine.GetFactory(MoreApplicationFactory.GetAssemblyName(effectiveDate));
        //    if (factory == null)
        //    {
        //        throw new Exception(
        //            "Either the assembly was not found.  Or it was compiled incorrectly. Recompile and try again.");
        //    }
        //    var ruleBook = factory.GetRuleBook(ruleBookName);

        //    if (ruleBook == null)
        //    {
        //        throw new Exception("Rule book not found.");
        //    }
        //    var result = ruleBook.Execute(inputs);
        //    if (result.Errors.Count() > 0)
        //    {
        //        var error = result.Errors.First();
        //        throw new Exception(string.Format("{0} - {1}", error.Message, error.RuleDescription));
        //    }
        //    return result.TraceInformation.ToArray();
        //}
        // GET api/values
        public IEnumerable<string> Get()
        {

            return new string[] { "value2" };
        }

        // GET api/values/5
        public string Get(RulesExecuteModel model)
        {
            return "value";
        }

        // POST api/values
        public object Post(RulesExecuteModel model)
        {

            // MoreApplicationFactory.GetCompiler(model.EffectiveDate);
            try
            {
                var inputs = FromJson(model.Data);
                var rulesEngine = new RulesEngine();
                var assemblyEffectiveDate = MoreApplicationFactory.GetAssemblyEffectiveDate(model.EffectiveDate);
                var factory = rulesEngine.GetFactory(MoreApplicationFactory.GetAssemblyName(assemblyEffectiveDate));
                if (factory == null)
                {
                    return
                        "Either the assembly was not found.  Or it was compiled incorrectly. Recompile and try again.";

                }
                var ruleBook = factory.GetRuleBook(model.RuleBook);

                if (ruleBook == null)
                {
                    return "Rule book not found.";
                }
                var result = ruleBook.Execute(inputs);
                if (result.Errors.Count() > 0)
                {
                    var error = result.Errors.First();
                    return string.Format("{0} - {1}", error.Message, error.RuleDescription);
                }
                if (model.Items != null && model.Items.Length > 0 && model.Items.First() != "null")
                {
                    return model.Items.ToDictionary(item => item, result.Find);
                }
                return GetJsonTrace(result.Results);
            }
            catch (Exception ex)
            {
                return ex.Message;
            }

        }

        // PUT api/values/5
        public void Put(int id, string value)
        {
        }

        // DELETE api/values/5
        public void Delete(int id)
        {
        }

    }
}